@@ -3,20 +3,31 @@
 # This installs all packages needed to build Shasta or to run
 # the dynamic executable. The static executable has no prerequisites.
 
+minimalInstall=false
+for arg in "$@"
+do
+    case $arg in
+        --minimal)
+        minimalInstall=true
+        shift
+        ;;
+        *)
+        echo "Usage: /path/to/InstallPrerequisites-Ubuntu.sh [--minimal]"
+        exit 1
+        ;;
+    esac
+done
+
 ubuntuPrettyName=$(cat /etc/os-release | grep PRETTY_NAME | cut -f2 -d'=')
 ubuntuVersion=$(cat /etc/os-release | grep VERSION_ID | cut -f2 -d'=')
 arch=$(uname -p)
 echo "Detected $ubuntuPrettyName running on $arch."
 
-isLatestOS=false
-if [ $ubuntuVersion == "\"20.04\"" ] ; then
-    isLatestOS=true
+isLatestOS=true
+if [ $ubuntuVersion \< "\"20.04\"" ] ; then
+    isLatestOS=false
 fi
 
-isOldOS=false
-if [ $ubuntuVersion \< "\"18.04\"" ] ; then
-    isOldOS=true
-fi
 
 isX86=false
 isArm=false
@@ -31,8 +42,8 @@ if [[ "$isX86" == false && "$isArm" == false ]]; then
     exit 1
 fi
 
-if [[ "$isArm" == true && "$isLatestOS" == false ]]; then
-    echo "Unsupported architecture & OS combination. Compiling on ARM requires Ubuntu 20.04 LTS or later."
+if [ "$isLatestOS" == false ]; then
+    echo "Unsupported OS. Building Shasta requires Ubuntu 20.04 LTS or later."
     exit 1
 fi
 
@@ -41,30 +52,23 @@ apt install -y git
 apt install -y g++
 apt install -y cmake
 apt install -y curl
-apt install -y libboost-all-dev
+apt install -y libboost-system-dev
+apt install -y libboost-program-options-dev
+apt install -y libboost-chrono-dev
 apt install -y libpng-dev
-apt install -y graphviz
-apt install -y gnuplot
-apt install -y ncbi-blast+
-apt install -y python3
-apt install -y python3-pip
-pip3 install pybind11
 
-
-if [[ "$isLatestOS" == true ]]; then
-    echo "Installing seqan v2.4.0 from Ubuntu repository."
-    apt install -y libseqan2-dev
-else
-    # SeqAn 2.4.0 is not available in the older Ubuntu repositories.
-    # Download it from GitHub, then install it.
-    echo "Installing seqan 2.4.0 from Github."
-    apt install -y curl
-    curl -L https://github.com/seqan/seqan/releases/download/seqan-v2.4.0/seqan-library-2.4.0.deb \
-        -o /dev/shm/seqan-library-2.4.0.deb
-    apt install /dev/shm/seqan-library-2.4.0.deb
-    rm /dev/shm/seqan-library-2.4.0.deb
+if [ "$minimalInstall" == false ]; then
+    # Install packages required for the HTTP server and Python-C++ bindings.
+    apt install -y libboost-graph-dev
+    apt install -y ncbi-blast+
+    apt install -y graphviz
+    apt install -y gnuplot
+    apt install -y python3
+    apt install -y python3-pip
+    pip3 install pybind11
 fi
 
+apt install -y libseqan2-dev
 
 # The spoa library is not available in the stable Ubuntu repository yet.
 # Download it from GitHub, then install it.
@@ -80,7 +84,7 @@ curl -L https://github.com/rvaser/spoa/releases/download/3.4.0/spoa-v3.4.0.tar.g
 tar -xvf spoa-v3.4.0.tar.gz
 
 spoaBuildFlags="-Dspoa_generate_dispatch=ON"
-if [[ "$isOldOS" == true || "$isArm" == true ]]; then
+if [[ "$isArm" == true ]]; then
     spoaBuildFlags="-Dspoa_generate_dispatch=OFF -Dspoa_optimize_for_portability=OFF -Dspoa_optimize_for_native=OFF"
 fi
 
@@ -104,7 +108,11 @@ rm -rf $tmpDirectoryName
 echo "=============================="
 echo " Relevant software versions"
 echo "=============================="
-echo "$(apt list git g++ cmake curl libseqan2-dev libboost-all-dev libpng-dev graphviz gnuplot ncbi-blast+ python3 python3-pip 2>/dev/null)"
+if [ "$minimalInstall" == false ]; then
+    echo "$(apt list git g++ cmake curl libseqan2-dev libboost-system-dev libpng-dev graphviz gnuplot ncbi-blast+ python3 python3-pip 2>/dev/null)"
+else
+    echo "$(apt list git g++ cmake curl libseqan2-dev libboost-system-dev 2>/dev/null)"
+fi
 echo "=============================="
 
 # Make sure the newly created libraries are immediately visible to the loader.
