@@ -6,9 +6,7 @@
 #include "AlignmentCandidates.hpp"
 #include "AssembledSegment.hpp"
 #include "AssemblyGraph.hpp"
-#include "ConflictReadGraph.hpp"
 #include "Coverage.hpp"
-#include "DirectedReadGraph.hpp"
 #include "dset64-gccAtomic.hpp"
 #include "Histogram.hpp"
 #include "HttpServer.hpp"
@@ -20,7 +18,6 @@
 #include "MemoryMappedObject.hpp"
 #include "MultithreadedObject.hpp"
 #include "OrientedReadPair.hpp"
-#include "PhasingData.hpp"
 #include "ReadGraph.hpp"
 #include "ReadFlags.hpp"
 #include "ReadId.hpp"
@@ -41,6 +38,7 @@ namespace shasta {
     class Alignment;
     class AlignmentGraph;
     class AlignmentInfo;
+    class Align4Options;
     class AssemblerOptions;
     class AssembledSegment;
     class CompressedAssemblyGraph;
@@ -48,7 +46,6 @@ namespace shasta {
     class LocalAssemblyGraph;
     class LocalAlignmentGraph;
     class LocalReadGraph;
-    class SegmentGraph;
     class Reads;
 
 #ifdef SHASTA_HTTP_SERVER
@@ -302,10 +299,6 @@ public:
         int bandExtend,
         int maxBand,
 
-        // The method used to create the read graph.
-        // This affects which alignments are used to create the marker graph.
-        int readGraphCreationMethod,
-
         // Minimum coverage (number of markers) for a vertex
         // of the marker graph to be kept.
         size_t minCoverage,
@@ -777,6 +770,35 @@ private:
         AlignmentInfo&);
 
 
+
+    // Member functions that use alignment algorithm 4.
+public:
+
+    // Python-callable version.
+    void alignOrientedReads4(
+        ReadId, Strand,
+        ReadId, Strand,
+        uint64_t m,
+        uint64_t deltaX,
+        uint64_t deltaY,
+        int64_t matchScore,
+        int64_t mismatchScore,
+        int64_t gapScore) const;
+
+    // Align two reads using alignment method 4.
+    // If debug is true, detailed output to html is produced.
+    // Otherwise, html is not used.
+    void alignOrientedReads4(
+        OrientedReadId,
+        OrientedReadId,
+        const Align4Options&,
+        Alignment&,
+        AlignmentInfo&,
+        bool debug) const;
+
+private:
+
+
     // Create a local alignment graph starting from a given oriented read
     // and walking out a given distance on the global alignment graph.
     // An alignment graph is an undirected graph in which each vertex
@@ -908,12 +930,8 @@ private:
 
 
     // Analyze the stored alignments involving a given oriented read.
-public:
-    void analyzeAlignments(ReadId, Strand) const;
 private:
     void analyzeAlignments1(ReadId, Strand) const;
-    void analyzeAlignments2(ReadId, Strand) const;
-    void analyzeAlignments3(ReadId, Strand) const;
 
 
     // Read graph and related functions and data.
@@ -1028,7 +1046,6 @@ private:
             uint32_t maxDistance,   // How far to go from starting oriented read.
             bool allowChimericReads,
             bool allowCrossStrandEdges,
-            size_t maxTrim,         // Used to define containment.
             double timeout,         // Or 0 for no timeout.
             LocalReadGraph&);
 
@@ -1040,7 +1057,6 @@ private:
         uint32_t maxDistance,   // How far to go from starting oriented read.
         bool allowChimericReads,
         bool allowCrossStrandEdges,
-        size_t maxTrim,         // Used to define containment.
         double timeout,         // Or 0 for no timeout.
         LocalReadGraph&);
 
@@ -1048,23 +1064,6 @@ public:
 
 
 
-    // Directed version of the read graph.
-    // This is only used if--ReadGraph.creationMethod is 1
-    // If --ReadGraph.creationMethod is 0, this is not used
-    // and instead we use the undirected read graph defined above.
-    DirectedReadGraph directedReadGraph;
-    void createDirectedReadGraph(
-        uint64_t maxTrim,
-        uint64_t containedNeighborCount,
-        uint64_t uncontainedNeighborCountPerDirection);
-    void accessDirectedReadGraphReadOnly();
-    void accessDirectedReadGraphReadWrite();
-    void markDirectedReadGraphConflictEdges1();
-    void markDirectedReadGraphConflictEdges2(int radius);
-    void markDirectedReadGraphConflictEdges3(int radius);
-
-
-
     // Write a FASTA file containing all reads that appear in
     // the local read graph.
     void writeLocalReadGraphReads(
@@ -1109,7 +1108,6 @@ private:
         double downsamplingFactor;
         int bandExtend;
         int maxBand;
-        int readGraphCreationMethod;
         uint32_t maxMarkerFrequency;
 
 
@@ -1207,9 +1205,6 @@ public:
     // Prune leaves from the strong subgraph of the global marker graph.
     void pruneMarkerGraphStrongSubgraph(size_t iterationCount);
 
-    // Analyze a vertex of the Marker graph.
-    void analyzeMarkerGraphVertex(MarkerGraph::VertexId) const;
-
     // Refine the marker graph by removing vertices in tangle regions,
     // then recreating edges. This must be called after
     // transitive reduction. After this is called, the only
@@ -1360,82 +1355,6 @@ private:
 
 
 
-    // Conflict read graph.
-    // See ConflictReadGraph.hpp for more information.
-public:
-    void createConflictReadGraph(
-        uint64_t threadCount,
-        uint32_t maxOffsetSigma,
-        uint32_t maxTrim,
-        uint32_t maxSkip,
-        uint32_t minAlignedMarkerCount);
-    void accessConflictReadGraph();
-    // void colorConflictReadGraph();
-    void cleanupConflictReadGraph();
-private:
-    // void createConflictReadGraphThreadFunction1(size_t threadId);
-    void createConflictReadGraphThreadFunction2(size_t threadId);
-    void addConflictGraphEdges(
-        ReadId,
-        const InducedAlignmentCriteria&,
-        // Work areas.
-        vector<OrientedReadId>&,
-        vector<OrientedReadId>&,
-        vector<InducedAlignment>&,
-        vector<uint64_t>&
-        );
-    class CreateConflictReadGraphData {
-    public:
-        InducedAlignmentCriteria inducedAlignmentCriteria;
-    };
-    CreateConflictReadGraphData createConflictReadGraphData;
-    ConflictReadGraph conflictReadGraph;
-
-
-
-    // Class used by colorConflictReadGraph.
-    // It creates a priority_queue containing this type.
-    class ColorConflictReadGraphData {
-    public:
-        DirectedReadGraph::VertexId vertexId;
-        uint64_t conflictReadGraphDegree;
-        uint64_t directedReadGraphKeptDegree;
-
-        ColorConflictReadGraphData(
-            DirectedReadGraph::VertexId,
-            const DirectedReadGraph&,
-            const ConflictReadGraph&);
-
-        // Used for sorting, so values with low conflict degree come first.
-        bool operator <(const ColorConflictReadGraphData& that) const
-        {
-            return
-                conflictReadGraphDegree < that.conflictReadGraphDegree
-                or
-                (
-                    conflictReadGraphDegree == that.conflictReadGraphDegree
-                    and
-                    directedReadGraphKeptDegree > that.directedReadGraphKeptDegree
-                );
-        }
-
-        // Used for sorting, so values with low conflict degree
-        // are at the top of the queue.
-        bool operator>(const ColorConflictReadGraphData& that) const
-        {
-            return
-                conflictReadGraphDegree > that.conflictReadGraphDegree
-                or
-                (
-                    conflictReadGraphDegree == that.conflictReadGraphDegree
-                    and
-                    directedReadGraphKeptDegree < that.directedReadGraphKeptDegree
-                );
-        }
-    };
-
-
-
 #ifdef SHASTA_HTTP_SERVER
     // Extract a local subgraph of the global marker graph.
     bool extractLocalMarkerGraphUsingStoredConnectivity(
@@ -1567,6 +1486,7 @@ public:
     void accessAssemblyGraphOrientedReadsByEdge();
     void writeAssemblyGraph(const string& fileName) const;
     void findAssemblyGraphBubbles();
+    void pruneAssemblyGraph(uint64_t pruneLength);
 
     // Gather and write out all reads that contributed to
     // each assembly graph edge.
@@ -1844,51 +1764,31 @@ public:
 
 
 
-    // The segment graph is used to detangle and determine reachability
-    // in the assembly graph.
-    void createSegmentGraph();
-    void colorGfaBySegmentGraphChain(uint64_t chainId) const;
-    shared_ptr<SegmentGraph> segmentGraphPointer;
-
-
-
-
-    // Analyze paths and pseudo-paths of oriented reads.
-    void analyzeOrientedReadPaths() const;
-    void analyzeOrientedReadPathsThroughSegment(
-        AssemblyGraph::EdgeId segmentId);
-    void analyzeOrientedReadPathsThroughSegment(
-        AssemblyGraph::EdgeId segmentId,
-        vector<AssemblyGraph::EdgeId>& forwardChokePoints,
-        vector<AssemblyGraph::EdgeId>& backwardChokePoints,
-        bool debug);
-    void followOrientedReadPaths(
-        AssemblyGraph::EdgeId segmentId,
-        bool forward);
-    void findOrientedReadsOnAssemblyGraphEdge(
-        AssemblyGraph::EdgeId,
-        vector<OrientedReadId>&
-    ) const;
+    // Analyze pseudo-paths of oriented reads.
     void alignPseudoPaths(ReadId, Strand, ReadId, Strand);
 
 
 
-
     // Data and functions used for the http server.
     // This function puts the server into an endless loop
     // of processing requests.
-    void writeHtmlBegin(ostream&, bool navigation=true) const;
+    void writeHtmlBegin(ostream&) const;
     void writeHtmlEnd(ostream&) const;
     void writeAssemblySummary(ostream&);
     void writeAssemblySummaryBody(ostream&);
     void writeAssemblySummaryJson(ostream&);
     void writeAssemblyIndex(ostream&) const;
+    static void writeStyle(ostream& html);
+
+
+#ifdef SHASTA_HTTP_SERVER
+
     void writeNavigation(ostream&) const;
     void writeNavigation(
         ostream& html,
         const string& title,
         const vector<pair <string, string> >&) const;
-    static void writeStyle(ostream& html);
+
     static void writePngToHtml(
         ostream& html,
         const string& pngFileName,
@@ -1900,25 +1800,24 @@ public:
         int height,
         const string& gnuplotCommands);
 
-#ifdef SHASTA_HTTP_SERVER
-
     void fillServerFunctionTable();
     void processRequest(
         const vector<string>& request,
         ostream&,
         const BrowserInformation&) override;
-    void writeMakeAllTablesSelectable(ostream&) const;
     void exploreSummary(const vector<string>&, ostream&);
     void exploreRead(const vector<string>&, ostream&);
     void blastRead(const vector<string>&, ostream&);
     void exploreAlignments(const vector<string>&, ostream&);
     void exploreAlignment(const vector<string>&, ostream&);
-    void displayAlignmentMatrix(const vector<string>&, ostream&);
+    void alignSequencesInBaseRepresentation(const vector<string>&, ostream&);
+    void alignSequencesInMarkerRepresentation(const vector<string>&, ostream&);
     void exploreAlignmentGraph(const vector<string>&, ostream&);
     void exploreReadGraph(const vector<string>&, ostream&);
     void exploreUndirectedReadGraph(const vector<string>&, ostream&);
     void exploreDirectedReadGraph(const vector<string>&, ostream&);
     void exploreCompressedAssemblyGraph(const vector<string>&, ostream&);
+    static void addScaleSvgButtons(ostream&);
     class HttpServerData {
     public:
 
@@ -1964,6 +1863,9 @@ public:
         double vertexScalingFactor;
         bool vertexScalingFactorIsPresent;
         string vertexScalingFactorString() const;
+        double edgeThicknessScalingFactor;
+        bool edgeThicknessScalingFactorIsPresent;
+        string edgeThicknessScalingFactorString() const;
         double arrowScalingFactor;
         bool arrowScalingFactorIsPresent;
         string arrowScalingFactorString() const;
@@ -1996,6 +1898,7 @@ public:
         ostream& html
     );
 #endif
+    void writeMakeAllTablesCopyable(ostream&) const;
 
     // Do bulk sampling of reads and accumulate stats about their alignments
     void assessAlignments(const vector<string>& request, ostream& html);
@@ -2076,10 +1979,7 @@ private:
         bool sizePixelsIsPresent;
         double timeout;
         bool timeoutIsPresent;
-        bool highlightedReadIdIsPresent;
-        ReadId highlightedReadId;
-        Strand highlightedStrand;
-        void writeForm(ostream&, AssemblyGraph::EdgeId edgeCount, bool allowHighlighting) const;
+        void writeForm(ostream&, AssemblyGraph::EdgeId edgeCount) const;
         bool hasMissingRequiredParameters() const;
     };
     void getLocalAssemblyGraphRequestParameters(
@@ -2110,32 +2010,6 @@ private:
 
 
 
-    // Functions and data structures used for phasing.
-public:
-    void createPhasingData(
-        size_t threadCount,
-        double phasingSimilarityThreshold,
-        int maxNeighborCount);
-    void accessPhasingData();
-    // double computePhasingSimilarity(OrientedReadId, OrientedReadId);
-    // double computePhasingSimilarity(ReadId, Strand, ReadId, Strand);
-    double computePhasingSimilarity(AssemblyGraph::EdgeId, AssemblyGraph::EdgeId);
-    uint64_t countCommonInternalOrientedReads(AssemblyGraph::EdgeId, AssemblyGraph::EdgeId);
-private:
-    PhasingData phasingData;
-    void phasingGatherOrientedReads(size_t threadCount);
-    void phasingGatherOrientedReadsPass1(size_t threadId);
-    void phasingGatherOrientedReadsPass2(size_t threadId);
-    void phasingGatherOrientedReadsPass(int pass);
-    void phasingGatherAssemblyGraphEdges(size_t threadCount);
-    void phasingGatherAssemblyGraphEdgesPass1(size_t threadId);
-    void phasingGatherAssemblyGraphEdgesPass2(size_t threadId);
-    void phasingGatherAssemblyGraphEdgesPass(int pass);
-    void phasingSortAssemblyGraphEdges(size_t threadCount);
-    void phasingSortAssemblyGraphEdgesThreadFunction(size_t threadId);
-    void phasingWriteBipartiteGraph();
-    void phasingFindSimilarForks();
-
 public:
     void test();
 };
