@@ -1,15 +1,15 @@
-// PngImage.hpp must be included first because of png issues on Ubuntu 16.04.
-#include "PngImage.hpp"
+#ifdef SHASTA_HTTP_SERVER
+
 
 // Shasta.
 #include "Assembler.hpp"
 #include "AssemblerOptions.hpp"
 #include "AlignmentGraph.hpp"
+#include "Histogram.hpp"
 #include "LocalAlignmentGraph.hpp"
 #include "platformDependent.hpp"
+#include "PngImage.hpp"
 #include "ReadId.hpp"
-#include "Histogram.hpp"
-
 using namespace shasta;
 
 // Boost libraries.
@@ -18,17 +18,15 @@ using namespace shasta;
 #include <boost/uuid/uuid_io.hpp>
 
 // Seqan
-#ifdef SHASTA_HTTP_SERVER
 #include <seqan/align.h>
-#endif
-
-#ifdef SHASTA_HTTP_SERVER
-
 
+// Standard library.
+#include "chrono.hpp"
 using std::random_device;
 using std::uniform_int_distribution;
 
 
+
 void Assembler::exploreAlignments(
     const vector<string>& request,
     ostream& html)
@@ -351,6 +349,14 @@ void Assembler::exploreAlignment(
     int maxBand = httpServerData.assemblerOptions->alignOptions.maxBand;
     getParameterValue(request, "maxBand", maxBand);
 
+    string displayMatrixString;
+    bool displayMatrix = getParameterValue(request, "displayMatrix", displayMatrixString);
+    int64_t markersPerPixel = 1;
+    getParameterValue(request, "markersPerPixel", markersPerPixel);
+    uint64_t magnifyFactor = 1;
+    getParameterValue(request, "magnifyFactor", magnifyFactor);
+    string displayDetailsString;
+    bool displayDetails = getParameterValue(request, "displayDetails", displayDetailsString);
 
 
     // Write the form.
@@ -387,7 +393,16 @@ void Assembler::exploreAlignment(
         html
     );
 
-    html << "</form>";
+    html <<
+        "<p><input type=checkbox name=displayMatrix" << (displayMatrix ? " checked=checked" : "") <<
+        "> Display alignment matrix at "
+        "<input type=text name=markersPerPixel size=6 value=" << markersPerPixel <<
+        "> markers per pixel and with each pixel magnified "
+        " <input type=text name=magnifyFactor size=6 value=" << magnifyFactor <<
+        "> times."
+        "<br><input type=checkbox name=displayDetails" << (displayDetails ? " checked=checked" : "") <<
+        "> Display alignment details"
+        "</form>";
 
 
     // If the readId's or strand's are missing, stop here.
@@ -418,14 +433,13 @@ void Assembler::exploreAlignment(
         getMarkersSortedByKmerId(orientedReadId0, markersSortedByKmerId[0]);
         getMarkersSortedByKmerId(orientedReadId1, markersSortedByKmerId[1]);
         AlignmentGraph graph;
-        const bool debug = true;
+        const bool debug = false;
         alignOrientedReads(
             markersSortedByKmerId,
             maxSkip, maxDrift, maxMarkerFrequency, debug, graph, alignment, alignmentInfo);
 
         if(alignment.ordinals.empty()) {
             html << "<p>The alignment is empty (it has no markers).";
-            return;
         }
     } else if(method == 1) {
         alignOrientedReads1(
@@ -442,41 +456,17 @@ void Assembler::exploreAlignment(
     }
 
 
-    // Make sure we have Alignment.png to display.
-    if(method != 0) {
-        vector<MarkerWithOrdinal> sortedMarkers0;
-        vector<MarkerWithOrdinal> sortedMarkers1;
-        getMarkersSortedByKmerId(orientedReadId0, sortedMarkers0);
-        getMarkersSortedByKmerId(orientedReadId1, sortedMarkers1);
-        AlignmentGraph::writeImage(
-            sortedMarkers0,
-            sortedMarkers1,
-            alignment,
-            "Alignment.png");
-    }
 
     if (alignment.ordinals.empty()) {
         html << "<p>The computed alignment is empty.";
-        return;
     }
     if (alignment.ordinals.size() < minAlignedMarkerCount) {
         html << "<p>Alignment has fewer than " << minAlignedMarkerCount << " markers.";
-        return;
     }
     if (alignmentInfo.minAlignedFraction() < minAlignedFraction) {
         html << "<p>Min aligned fraction is smaller than " << minAlignedFraction << ".";
-        return;
     }
 
-    // If the alignment has too much trim, skip it.
-    uint32_t leftTrim;
-    uint32_t rightTrim;
-    tie(leftTrim, rightTrim) = alignmentInfo.computeTrim();
-    if(leftTrim>maxTrim || rightTrim>maxTrim) {
-        html << "<p>Alignment has too much trim. Left trim = " << leftTrim
-            << " Right trim = " << rightTrim;
-        return;
-    }
 
 
     // Write summary information for this alignment.
@@ -486,99 +476,152 @@ void Assembler::exploreAlignment(
         orientedReadId1,
         alignmentInfo,
         html);
-    html << "<br>See below for alignment details.";
 
 
-    // Create a base64 version of the png file.
-    const string command = "base64 Alignment.png > Alignment.png.base64";
-    ::system(command.c_str());
 
+    if(displayMatrix) {
 
-    // Write out the picture with the alignment.
-    html <<
-        "<h3>Alignment matrix</h3>"
-        "<p>In the picture, horizontal positions correspond to marker ordinals on " <<
-        orientedReadId0 << " (marker 0 is on left) "
-        "and vertical positions correspond to marker ordinals on " <<
-        orientedReadId1 << " (marker 0 is on top). "
-        "Each faint line corresponds to 10 markers."
-        "<p><img id=\"alignmentMatrix\" onmousemove=\"updateTitle(event)\" "
-        "src=\"data:image/png;base64,";
-    ifstream png("Alignment.png.base64");
-    html << png.rdbuf();
-    html << "\"/>"
-        "<script>"
-        "function updateTitle(e)"
-        "{"
-        "    var element = document.getElementById(\"alignmentMatrix\");"
-        "    var rectangle = element.getBoundingClientRect();"
-        "    var x = e.clientX - Math.round(rectangle.left);"
-        "    var y = e.clientY - Math.round(rectangle.top);"
-        "    element.title = " <<
-        "\"" << orientedReadId0 << " marker \" + x + \", \" + "
-        "\"" << orientedReadId1 << " marker \" + y;"
-        "}"
-        "</script>";
+        // Create an image of the alignment matrix in Alignment.png.
+        vector<MarkerWithOrdinal> sortedMarkers0;
+        vector<MarkerWithOrdinal> sortedMarkers1;
+        getMarkersSortedByKmerId(orientedReadId0, sortedMarkers0);
+        getMarkersSortedByKmerId(orientedReadId1, sortedMarkers1);
+        AlignmentGraph::writeImage(
+            sortedMarkers0,
+            sortedMarkers1,
+            alignment,
+            markersPerPixel,
+            magnifyFactor,
+            "Alignment.png");
+
+        // Create a base64 version of the png file.
+        const string command = "base64 Alignment.png > Alignment.png.base64";
+        ::system(command.c_str());
+
+        // Write out the picture with the alignment.
+        html <<
+            "<h3>Alignment matrix</h3>"
+            "<p>In the picture, horizontal positions correspond to marker ordinals on " <<
+            orientedReadId0 << " (marker 0 is on left) "
+            "and vertical positions correspond to marker ordinals on " <<
+            orientedReadId1 << " (marker 0 is on top). "
+            "Each faint line corresponds to 10 markers."
+            "<p><img id=\"alignmentMatrix\" onmousemove=\"updateTitle(event)\" "
+            "src=\"data:image/png;base64,";
+        ifstream png("Alignment.png.base64");
+        html << png.rdbuf();
+
+        // Adjust the tooltip dynamically to follow the mouse.
+        html << "\"/>"
+            "<script>"
+            "function updateTitle(e)"
+            "{"
+            "    var element = document.getElementById(\"alignmentMatrix\");"
+            "    var rectangle = element.getBoundingClientRect();"
+            "    var x = Math.round((" << markersPerPixel << " * (e.clientX - Math.round(rectangle.left))) / " << magnifyFactor << ");"
+            "    var y = Math.round((" << markersPerPixel << " * (e.clientY - Math.round(rectangle.top))) / " << magnifyFactor << ");"
+            "    element.title = " <<
+            "\"" << orientedReadId0 << " marker \" + x + \", \" + "
+            "\"" << orientedReadId1 << " marker \" + y;"
+            "}"
+            "</script>";
+    }
 
 
 
     // Write out details of the alignment.
-    html <<
-        "<h3>Alignment details</h3>"
-        "<table>"
+    if(displayDetails) {
 
-        "<tr>"
-        "<th rowspan=2>K-mer"
-        "<th colspan=3>Ordinals"
-        "<th colspan=2>Positions<br>(RLE)"
-
-        "<tr>"
-        "<th>" << orientedReadId0 <<
-        "<th>" << orientedReadId1 <<
-        "<th>Offset"
-        "<th>" << orientedReadId0 <<
-        "<th>" << orientedReadId1;
-
-    const auto markers0 = markers[orientedReadId0.getValue()];
-    const auto markers1 = markers[orientedReadId1.getValue()];
-    for(const auto& ordinals: alignment.ordinals) {
-        const auto ordinal0 = ordinals[0];
-        const auto ordinal1 = ordinals[1];
-        const auto& marker0 = markers0[ordinal0];
-        const auto& marker1 = markers1[ordinal1];
-        const auto kmerId = marker0.kmerId;
-        SHASTA_ASSERT(marker1.kmerId == kmerId);
-        const Kmer kmer(kmerId, assemblerInfo->k);
-
-        html << "<tr><td style='font-family:monospace'>";
-        kmer.write(html, assemblerInfo->k);
+        // To make sure cut and paste to a spreadsheet works,
+        // keep the header simple (no colspan, rowspan).
         html <<
+            "<h3>Alignment details</h3>"
+            "<table>"
 
-            "<td class=centered>"
-            "<a href=\"exploreRead?readId=" << orientedReadId0.getReadId() <<
-            "&amp;strand=" << orientedReadId0.getStrand() <<
-            "&amp;highlightMarker=" << ordinal0 <<
-            "#" << ordinal0 << "\">" << ordinal0 << "</a>"
+            "<tr>"
+            "<th>K-mer"
+            "<th style='background-color:AliceBlue' title='Marker ordinal on " <<
+            orientedReadId0 << "'>Ord<br>" << orientedReadId0 <<
+            "<th style='background-color:AliceBlue' title='Marker ordinal on " <<
+            orientedReadId1 << "'>Ord<br>" << orientedReadId1 <<
+            "<th style='background-color:AliceBlue' title='Marker ordinal offset'>Ord<br>Offset"
+            "<th style='background-color:CornSilk' title='RLE position on " <<
+            orientedReadId0 << "'>RLE<br>" << orientedReadId0 <<
+            "<th style='background-color:CornSilk' title='RLE position on " <<
+            orientedReadId1 << "'>RLE<br>" << orientedReadId1 <<
+            "<th style='background-color:CornSilk' title='RLE position offset'>RLE<br>Offset"
+            "<th style='background-color:LavenderBlush' title='Raw position on " <<
+            orientedReadId0 << "'>Raw<br>" << orientedReadId0 <<
+            "<th style='background-color:LavenderBlush' title='Raw position on " <<
+            orientedReadId1 << "'>Raw<br>" << orientedReadId1 <<
+            "<th style='background-color:LavenderBlush' title='Raw position offset'>Raw<br>Offset";
 
-            "<td class=centered>"
-            "<a href=\"exploreRead?readId=" << orientedReadId1.getReadId() <<
-            "&amp;strand=" << orientedReadId1.getStrand() <<
-            "&amp;highlightMarker=" << ordinal1 <<
-            "#" << ordinal1 << "\">" << ordinal1 << "</a>"
+        // Access the markers for the two oriented reads.
+        const auto markers0 = markers[orientedReadId0.getValue()];
+        const auto markers1 = markers[orientedReadId1.getValue()];
 
-            "<td class=centered>" << int32_t(ordinal0) - int32_t(ordinal1) <<
-            "<td class=centered>" << marker0.position <<
-            "<td class=centered>" << marker1.position;
+        // Compute the raw position corresponding to each RLE position.
+        const vector<uint32_t> rawPositions0 = reads->getRawPositions(orientedReadId0);
+        const vector<uint32_t> rawPositions1 = reads->getRawPositions(orientedReadId1);
 
+        // Loop over all markers.
+        for(const auto& ordinals: alignment.ordinals) {
+            const auto ordinal0 = ordinals[0];
+            const auto ordinal1 = ordinals[1];
+
+            const auto& marker0 = markers0[ordinal0];
+            const auto& marker1 = markers1[ordinal1];
+
+            const auto kmerId = marker0.kmerId;
+            SHASTA_ASSERT(marker1.kmerId == kmerId);
+            const Kmer kmer(kmerId, assemblerInfo->k);
+
+            const uint32_t rlePosition0 = marker0.position;
+            const uint32_t rlePosition1 = marker1.position;
+
+            const uint32_t rawPosition0 = rawPositions0[rlePosition0];
+            const uint32_t rawPosition1 = rawPositions1[rlePosition1];
+
+            html << "<tr><td style='font-family:monospace'>";
+            kmer.write(html, assemblerInfo->k);
+            html <<
+
+                // Ordinal0.
+                "<td class=centered style='background-color:AliceBlue'>"
+                "<a href=\"exploreRead?readId=" << orientedReadId0.getReadId() <<
+                "&amp;strand=" << orientedReadId0.getStrand() <<
+                "&amp;highlightMarker=" << ordinal0 <<
+                "#" << ordinal0 << "\">" << ordinal0 << "</a>"
+
+                // Ordinal1.
+                "<td class=centered style='background-color:AliceBlue'>"
+                "<a href=\"exploreRead?readId=" << orientedReadId1.getReadId() <<
+                "&amp;strand=" << orientedReadId1.getStrand() <<
+                "&amp;highlightMarker=" << ordinal1 <<
+                "#" << ordinal1 << "\">" << ordinal1 << "</a>"
+
+                // Ordinal offset.
+                "<td class=centered style='background-color:AliceBlue'>" << int32_t(ordinal0) - int32_t(ordinal1) <<
+
+                // RLE positions and their offset.
+                "<td class=centered style='background-color:CornSilk'>" << rlePosition0 <<
+                "<td class=centered style='background-color:CornSilk'>" << rlePosition1 <<
+                "<td class=centered style='background-color:CornSilk'>" << int32_t(rlePosition0) - int32_t(rlePosition1) <<
+
+                // Raw positions and their offset.
+                "<td class=centered style='background-color:LavenderBlush'>" << rawPosition0 <<
+                "<td class=centered style='background-color:LavenderBlush'>" << rawPosition1 <<
+                "<td class=centered style='background-color:LavenderBlush'>" << int32_t(rawPosition0) - int32_t(rawPosition1);
+
+        }
+
+        html << "</table>";
     }
-
-    html << "</table>";
 }
 
 
 
-// Display a base-by-base alignment matrix between two given sequences.
-void Assembler::displayAlignmentMatrix(
+void Assembler::alignSequencesInBaseRepresentation(
     const vector<string>& request,
     ostream& html)
 {
@@ -614,11 +657,9 @@ void Assembler::displayAlignmentMatrix(
     const bool showGrid = (showGridString == "on");
 
 
-    // Get the zoom factor.
-
     // Write the form.
     html <<
-        "<p>Display a base-by-base alignment of these two sequences:"
+        "<p>Align these two sequences in base representation:"
         "<form>"
         "<input style='font-family:monospace' type=text name=sequence0 required size=64 value='" << sequenceString0 << "'>"
         "<br><input style='font-family:monospace' type=text name=sequence1 required size=64 value='" << sequenceString1 << "'>"
@@ -627,7 +668,7 @@ void Assembler::displayAlignmentMatrix(
         "<br><input type=checkbox name=showAlignment" << (showAlignment ? " checked" : "") << "> Show the alignment and highlight it in the alignment matrix."
         "<br><input type=checkbox name=showGrid" << (showGrid ? " checked" : "") << "> Show a grid on the alignment matrix."
         "<br>Zoom factor: <input type=text name=zoom required value=" << zoom << ">"
-        "<br><input type=submit value='Display'>"
+        "<br><input type=submit value='Align'>"
         "</form>";
 
     // If either sequence is empty, do nothing.
@@ -846,14 +887,14 @@ void Assembler::displayAlignmentMatrix(
         html << "<br>Sequence lengths: " << n0 << " " << n1 <<
             "<br>Optimal alignment has length " << alignmentLength <<
             ", score " << score <<
-            ":<div style='font-family:monospace'>";
+            "<pre style='font-family:monospace'>";
         for(size_t i=0; i<2; i++) {
-            html << "<br>";
             for(int j=0; j<alignmentLength; j++) {
                 html << alignment[i][j];
             }
+            html << "\n";
         }
-        html << "</div>";
+        html << "</pre>";
 
 
         int i0 = 0;
@@ -909,6 +950,22 @@ void Assembler::displayAlignmentMatrix(
 #endif
 }
 
+
+void Assembler::alignSequencesInMarkerRepresentation(
+    const vector<string>& request,
+    ostream& html)
+{
+    html <<
+        "<h1>Align two sequences in marker representation</h1>"
+        "<form method=post enctype='multipart/form-data'>"
+        "<input style='font-family:monospace' type=text name=sequence0 required size=64>"
+        "<input style='font-family:monospace' type=text name=sequence1 required size=64>"
+        "<br><input type=submit value='Align'>"
+        "</form>";
+}
+
+
+
 void Assembler::renderEditableAlignmentConfig(
     const int method,
     const uint64_t maxSkip,
@@ -1105,7 +1162,7 @@ void Assembler::computeAllAlignments(
     computeAllAlignmentsData.orientedReadId0 = orientedReadId0;
     const size_t threadCount =std::thread::hardware_concurrency();
     computeAllAlignmentsData.threadAlignments.resize(threadCount);
-    const size_t batchSize = 1000;
+    const size_t batchSize = 1;
     setupLoadBalancing(reads->readCount(), batchSize);
     const auto t0 = std::chrono::steady_clock::now();
     runThreads(&Assembler::computeAllAlignmentsThreadFunction, threadCount);
@@ -1958,7 +2015,7 @@ void Assembler::exploreAlignmentGraph(
     uint32_t maxDistance = 2;
     getParameterValue(request, "maxDistance", maxDistance);
 
-    uint32_t sizePixels = 1200;
+    uint32_t sizePixels = 600;
     getParameterValue(request, "sizePixels", sizePixels);
 
     double timeout= 30;
@@ -2056,89 +2113,32 @@ void Assembler::exploreAlignmentGraph(
 
     // Create the local alignment graph.
     LocalAlignmentGraph graph;
-    const auto createStartTime = steady_clock::now();
     if(!createLocalAlignmentGraph(orientedReadId,
         minAlignedMarkerCount, maxTrim, maxDistance, timeout, graph)) {
         html << "<p>Timeout for graph creation exceeded. Increase the timeout or reduce the maximum distance from the start vertex.";
         return;
     }
-    const auto createFinishTime = steady_clock::now();
-
-    // Write it out in graphviz format.
-    const string uuid = to_string(boost::uuids::random_generator()());
-    const string dotFileName = tmpDirectory() + uuid + ".dot";
-    graph.write(dotFileName, maxDistance);
-
-    // Compute layout in svg format.
-    const string command =
-        timeoutCommand() + " " + to_string(timeout - seconds(createFinishTime - createStartTime)) +
-        " sfdp -O -T svg " + dotFileName +
-        " -Gsize=" + to_string(sizePixels/72.);
-    const auto layoutStartTime = steady_clock::now();
-    const int commandStatus = ::system(command.c_str());
-    const auto layoutFinishTime = steady_clock::now();
-    if(WIFEXITED(commandStatus)) {
-        const int exitStatus = WEXITSTATUS(commandStatus);
-        if(exitStatus == 124) {
-            html << "<p>Timeout for graph layout exceeded. Increase the timeout or reduce the maximum distance from the start vertex.";
-            filesystem::remove(dotFileName);
-            return;
-        }
-        else if(exitStatus!=0 && exitStatus!=1) {    // sfdp returns 1 all the time just because of the message about missing triangulation.
-            filesystem::remove(dotFileName);
-            throw runtime_error("Error " + to_string(exitStatus) + " running graph layout command: " + command);
-        }
-    } else if(WIFSIGNALED(commandStatus)) {
-        const int signalNumber = WTERMSIG(commandStatus);
-        throw runtime_error("Signal " + to_string(signalNumber) + " while running graph layout command: " + command);
-    } else {
-        throw runtime_error("Abnormal status " + to_string(commandStatus) + " while running graph layout command: " + command);
-
-    }
-    // Remove the .dot file.
-    filesystem::remove(dotFileName);
 
 
 
-    // Write a title and display the graph.
+    // Write a title.
     html <<
-        "<h1 style='line-height:10px'>Alignment graph near oriented read " << orientedReadId << "</h1>"
-        "Color legend: "
-        "<span style='background-color:LightGreen'>start vertex</span> "
-        "<span style='background-color:cyan'>vertices at maximum distance (" << maxDistance <<
-        ") from the start vertex</span>.";
-
-
-    // Display the graph.
-    const string svgFileName = dotFileName + ".svg";
-    ifstream svgFile(svgFileName);
-    html << svgFile.rdbuf();
-    svgFile.close();
-
-
-
-    // Add to each vertex a cursor that shows you can click on it.
-    html <<
-        "<script>"
-        "var vertices = document.getElementsByClassName('node');"
-        "for (var i=0;i<vertices.length; i++) {"
-        "    vertices[i].style.cursor = 'pointer';"
-        "}"
-        "</script>";
-
-
-
-    // Remove the .svg file.
-    filesystem::remove(svgFileName);
+        "<h1 style='line-height:10px'>Alignment graph near oriented read " << orientedReadId << "</h1>";
 
     // Write additional graph information.
     html <<
         "<br>This portion of the alignment graph has " << num_vertices(graph) <<
-        " vertices and " << num_edges(graph) << " edges." <<
-        "<br>Graph creation took " <<
-        std::setprecision(2) << seconds(createFinishTime-createStartTime) <<
-        " s.<br>Graph layout took " <<
-        std::setprecision(2) << seconds(layoutFinishTime-layoutStartTime) << " s.";
+        " vertices and " << num_edges(graph) << " edges.<br>";
+
+    // Buttons to resize the svg locally.
+    addScaleSvgButtons(html);
+
+    // Write the graph to svg directly, without using Graphviz rendering.
+    graph.computeLayout("sfdp", timeout);
+    const double vertexScalingFactor = 1.;
+    const double edgeThicknessScalingFactor = 1.;
+    graph.writeSvg("svg", sizePixels, sizePixels,
+        vertexScalingFactor, edgeThicknessScalingFactor, maxDistance, html);
 
     // Write a histogram of the number of vertices by distance.
     vector<int> histogram(maxDistance+1, 0);
