@@ -31,8 +31,7 @@ void Assembler::exploreAssemblyGraph(
     getLocalAssemblyGraphRequestParameters(request, requestParameters);
 
     // Write the form.
-    const bool allowHighlighting = phasingData.assemblyGraphEdges.isOpen();
-    requestParameters.writeForm(html, assemblyGraph.edges.size(), allowHighlighting);
+    requestParameters.writeForm(html, assemblyGraph.edges.size());
 
     // If any required values are missing, stop here.
     if(requestParameters.hasMissingRequiredParameters()) {
@@ -70,36 +69,6 @@ void Assembler::exploreAssemblyGraph(
     }
 
 
-#if 0
-    // Highlight edges containing the specified oriented read.
-    if(allowHighlighting && requestParameters.highlightedReadIdIsPresent) {
-
-        // Create a map of local edges, keyed by global edge id.
-        std::map<AssemblyGraph::EdgeId, LocalAssemblyGraph::edge_descriptor> edgeMap;
-        BGL_FORALL_EDGES(e, graph, LocalAssemblyGraph) {
-            edgeMap.insert(make_pair(graph[e].edgeId, e));
-        }
-
-        // Get the edges we need to highlight.
-        const OrientedReadId orientedReadId(
-            requestParameters.highlightedReadId,
-            requestParameters.highlightedStrand);
-        const span<AssemblyGraph::EdgeId> highlightedEdges =
-            phasingGraph.assemblyGraphEdges[orientedReadId.getValue()];
-
-        // Highlight those edges, if present in the local assembly graph.
-        for(const AssemblyGraph::EdgeId edgeId: highlightedEdges) {
-            const auto it = edgeMap.find(edgeId);
-            if(it == edgeMap.end()) {
-                continue;
-            }
-            const LocalAssemblyGraph::edge_descriptor e = it->second;
-            graph[e].isHighlighted = true;
-        }
-
-    }
-#endif
-
 
     // Write it out in graphviz format.
     const string uuid = to_string(boost::uuids::random_generator()());
@@ -113,11 +82,10 @@ void Assembler::exploreAssemblyGraph(
 
 
 
-    // Compute graph layout in svg format.
+    // Compute graph layout.
     const string command =
         timeoutCommand() + " " + to_string(requestParameters.timeout - seconds(createFinishTime - createStartTime)) +
-        " dot -O -T svg " + dotFileName +
-        " -Gsize=" + to_string(requestParameters.sizePixels/72.);
+        " dot -O -T svg " + dotFileName;
     const int commandStatus = ::system(command.c_str());
     if(WIFEXITED(commandStatus)) {
         const int exitStatus = WEXITSTATUS(commandStatus);
@@ -139,17 +107,30 @@ void Assembler::exploreAssemblyGraph(
     }
 
     // Remove the .dot file.
-    // filesystem::remove(dotFileName);
+    filesystem::remove(dotFileName);
+
+    // Buttons to resize the svg locally.
+    html << "<br>";
+    addScaleSvgButtons(html);
 
     // Copy the svg to html.
     const string svgFileName = dotFileName + ".svg";
     ifstream svgFile(svgFileName);
+    html << "<div id=svgDiv style='display:none'>"; // Make it invisible until after we scale it.
     html << "<br>" << svgFile.rdbuf();
     svgFile.close();
 
     // Remove the .svg file.
     filesystem::remove(svgFileName);
 
+    // Scale to desired size, then make it visible.
+    html <<
+        "</div>"
+        "<script>"
+        "var svgElement = document.getElementsByTagName('svg')[0];"
+        "svgElement.setAttribute('width', " << requestParameters.sizePixels << ");"
+        "document.getElementById('svgDiv').setAttribute('style', 'display:block');"
+        "</script>";
 }
 
 
@@ -188,20 +169,13 @@ void Assembler::getLocalAssemblyGraphRequestParameters(
     parameters.timeoutIsPresent = getParameterValue(
         request, "timeout", parameters.timeout);
 
-    parameters.highlightedReadIdIsPresent = getParameterValue(
-        request, "highlightedReadId", parameters.highlightedReadId);
-
-    parameters.highlightedStrand = 0;
-    getParameterValue(request, "highlightedStrand", parameters.highlightedStrand);
-
 }
 
 
 
 void Assembler::LocalAssemblyGraphRequestParameters::writeForm(
     ostream& html,
-    AssemblyGraph::EdgeId edgeCount,
-    bool allowHighlighting) const
+    AssemblyGraph::EdgeId edgeCount) const
 {
     html <<
         "<h3>Display a local subgraph of the global assembly graph</h3>"
@@ -251,26 +225,7 @@ void Assembler::LocalAssemblyGraphRequestParameters::writeForm(
         "<td>Timeout (seconds) for graph creation and layout"
         "<td><input type=text required name=timeout size=8 style='text-align:center'"
         << (timeoutIsPresent ? (" value='" + to_string(timeout)+"'") : " value='30'") <<
-        ">";
-
-    if(allowHighlighting) {
-        html <<
-            "<tr>"
-            "<td>Oriented read to be highlighted"
-            "<td><input type=text name=highlightedReadId size=8 style='text-align:center'";
-
-        if(highlightedReadIdIsPresent) {
-            html << " value='" << highlightedReadId << "'";
-        }
-        html << ">";
-
-        html << "<tr><td>Strand of oriented read to be highlighted><td class=centered>";
-        writeStrandSelection(html, "highlightedStrand", highlightedStrand==0, highlightedStrand==1);
-
-
-    }
-
-    html <<
+        ">"
         "</table>"
 
         "<br><input type=submit value='Display'>"
@@ -360,20 +315,6 @@ void Assembler::exploreAssemblyGraphEdge(const vector<string>& request, ostream&
 
 
 
-    // Phasing information.
-    if (phasingData.orientedReads.isOpen()) {
-        const span<OrientedReadId> orientedReadIds =
-            phasingData.orientedReads[edgeId];
-        html << "<p>The following oriented reads are internal to the this "
-            "assembly graph edge:<br>";
-        for(const OrientedReadId orientedReadId: orientedReadIds) {
-            html << orientedReadId << " ";
-        }
-    }
-
-
-
-
     // If this edge was not assembled, point to the reverse
     // complemented edge, which was assembled.
     if(!assemblyGraph.isAssembledEdge(edgeId)) {
