@@ -108,13 +108,13 @@ void Assembler::exploreMarkerGraph(
         requestParameters.addLabels,
         requestParameters.useDotLayout,
         requestParameters.vertexScalingFactor,
+        requestParameters.edgeThicknessScalingFactor,
         requestParameters.arrowScalingFactor);
 
     // Compute layout in svg format.
     const string command =
         timeoutCommand() + " " + to_string(requestParameters.timeout - int(seconds(createFinishTime - createStartTime))) +
-        " dot -O -T svg " + dotFileName +
-        " -Gsize=" + to_string(requestParameters.sizePixels/72.);
+        " dot -O -T svg " + dotFileName;
     const int commandStatus = ::system(command.c_str());
     if(WIFEXITED(commandStatus)) {
         const int exitStatus = WEXITSTATUS(commandStatus);
@@ -135,7 +135,7 @@ void Assembler::exploreMarkerGraph(
 
     }
     // Remove the .dot file.
-    // filesystem::remove(dotFileName);
+    filesystem::remove(dotFileName);
 
 
 
@@ -144,14 +144,27 @@ void Assembler::exploreMarkerGraph(
         "<h2>Marker graph near marker graph vertex " << requestParameters.vertexId <<
         "</h2>";
 
+    // Buttons to resize the svg locally.
+    addScaleSvgButtons(html);
+
     const string svgFileName = dotFileName + ".svg";
     ifstream svgFile(svgFileName);
+    html << "<div id=svgDiv style='display:none'>"; // Make it invisible until after we scale it.
     html << svgFile.rdbuf();
     svgFile.close();
 
     // Remove the .svg file.
     filesystem::remove(svgFileName);
 
+    // Scale to desired size, then make it visible.
+    html <<
+        "</div>"
+        "<script>"
+        "var svgElement = document.getElementsByTagName('svg')[0];"
+        "svgElement.setAttribute('width', " << requestParameters.sizePixels << ");"
+        "document.getElementById('svgDiv').setAttribute('style', 'display:block');"
+        "</script>";
+
 
 
     // Make the vertices clickable: left click recenters
@@ -274,6 +287,10 @@ void Assembler::getLocalMarkerGraphRequestParameters(
     parameters.vertexScalingFactorIsPresent = getParameterValue(
         request, "vertexScalingFactor", parameters.vertexScalingFactor);
 
+    parameters.edgeThicknessScalingFactor = 1;
+    parameters.edgeThicknessScalingFactorIsPresent = getParameterValue(
+        request, "edgeThicknessScalingFactor", parameters.edgeThicknessScalingFactor);
+
     parameters.arrowScalingFactor = 1;
     parameters.arrowScalingFactorIsPresent = getParameterValue(
         request, "arrowScalingFactor", parameters.arrowScalingFactor);
@@ -357,12 +374,16 @@ void Assembler::LocalMarkerGraphRequestParameters::writeForm(
         "<td><input type=text required name=vertexScalingFactor size=8 style='text-align:center'" <<
         " value='" + vertexScalingFactorString() + "'>" <<
 
+        "<tr>"
+        "<td>Edge thickness scaling factor"
+        "<td><input type=text required name=edgeThicknessScalingFactor size=8 style='text-align:center'" <<
+        " value='" + edgeThicknessScalingFactorString() + "'>" <<
+
         "<tr>"
         "<td>Edge arrow scaling factor"
         "<td><input type=text required name=arrowScalingFactor size=8 style='text-align:center'" <<
         " value='" + arrowScalingFactorString() + "'>" <<
 
-
         "<tr title='Maximum time allowed (seconds) for graph creation and layout, or 0 if unlimited'>"
         "<td>Timeout (seconds) for graph creation and layout"
         "<td><input type=text required name=timeout size=8 style='text-align:center'"
@@ -414,6 +435,19 @@ string Assembler::LocalMarkerGraphRequestParameters::arrowScalingFactorString()
 
 
 
+string Assembler::LocalMarkerGraphRequestParameters::edgeThicknessScalingFactorString() const
+{
+    if(edgeThicknessScalingFactorIsPresent) {
+        std::ostringstream s;
+        s << edgeThicknessScalingFactor;
+        return s.str();
+    } else {
+        return "1";
+    }
+}
+
+
+
 void Assembler::exploreMarkerGraphVertex(const vector<string>& request, ostream& html)
 {
     // Get the vertex id.
