@@ -1,3 +1,5 @@
+#ifdef SHASTA_HTTP_SERVER
+
 // Shasta.
 #include "Assembler.hpp"
 #include "AssemblerOptions.hpp"
@@ -15,15 +17,12 @@ using namespace shasta;
 #include "chrono.hpp"
 #include "iterator.hpp"
 
-
 void Assembler::exploreReadGraph(
     const vector<string>& request,
     ostream& html)
 {
     if(readGraph.edges.isOpen && readGraph.connectivity.isOpen()) {
         exploreUndirectedReadGraph(request, html);
-    } else if(directedReadGraph.isOpen()) {
-        exploreDirectedReadGraph(request, html);
     } else {
         html << "The read graph is not available." << endl;
     }
@@ -43,8 +42,9 @@ bool parseCommaSeparatedReadIDs(string& commaSeparatedReadIds, vector<OrientedRe
                 token.clear();
             }
             catch(exception& e){
-                html << "<p>Invalid read id or read strand: '" << token << "'</p>";
-                html << "<p>" << e.what() << "</p>";
+                html << "<p>Invalid oriented read id: '" << token << "'</p>";
+                html << "<p>Specify one or more comma separated oriented read ids of the form ReadId-strand "
+                    "where strand is 0 or 1. For example: 757-1,1048-0";
                 return false;
             }
         }
@@ -60,8 +60,9 @@ bool parseCommaSeparatedReadIDs(string& commaSeparatedReadIds, vector<OrientedRe
             readIds.push_back(readId);
         }
         catch(exception& e){
-            html << "<p>Invalid read id or read strand: '" << token << "'</p>";
-            html << "<p>" << e.what() << "</p>";
+            html << "<p>Invalid oriented read id: '" << token << "'</p>";
+            html << "<p>Specify one or more comma separated oriented read ids of the form ReadId-strand "
+                "where strand is 0 or 1. For example: 757-1,1048-0";
             return false;
         }
     }
@@ -70,6 +71,37 @@ bool parseCommaSeparatedReadIDs(string& commaSeparatedReadIds, vector<OrientedRe
 }
 
 
+
+// Write to html buttons to resize the svg locally (in the browser).
+// This assumes that the page contains a single svg object.
+void Assembler::addScaleSvgButtons(ostream& html)
+{
+    html << R"stringDelimiter(
+        <script>
+        function svgLarger()
+        {
+            var element = document.getElementsByTagName("svg")[0];
+            element.setAttribute("width", 1.25*element.getAttribute("width"));
+            element.setAttribute("height", 1.25*element.getAttribute("height"));
+            document.getElementById("largerButton").focus();
+        }
+        function svgSmaller()
+        {
+            var element = document.getElementsByTagName("svg")[0];
+            element.setAttribute("width", 0.8*element.getAttribute("width"));
+            element.setAttribute("height", 0.8*element.getAttribute("height"));
+            document.getElementById("smallerButton").focus();
+        }
+        </script>
+        <button type="button" id=largerButton onclick='svgLarger()'>Larger</button>
+        &nbsp;
+        <button type="button" id=smallerButton onclick='svgSmaller()'>Smaller</button>
+        <br>
+    )stringDelimiter";    
+}
+
+
+
 void Assembler::exploreUndirectedReadGraph(
     const vector<string>& request,
     ostream& html) {
@@ -82,9 +114,6 @@ void Assembler::exploreUndirectedReadGraph(
     uint32_t maxDistance = 2;
     getParameterValue(request, "maxDistance", maxDistance);
 
-    uint32_t maxTrim = httpServerData.assemblerOptions->alignOptions.maxTrim;
-    getParameterValue(request, "maxTrim", maxTrim);
-
     string allowChimericReadsString;
     const bool allowChimericReads = getParameterValue(request, "allowChimericReads", allowChimericReadsString);
 
@@ -94,30 +123,21 @@ void Assembler::exploreUndirectedReadGraph(
     string layoutMethod = "sfdp";
     getParameterValue(request, "layoutMethod", layoutMethod);
 
-    uint32_t sizePixels = 1200;
+    uint32_t sizePixels = 600;
     getParameterValue(request, "sizePixels", sizePixels);
 
-    double vertexScalingFactor = 2.;
+    double vertexScalingFactor = 1.;
     getParameterValue(request, "vertexScalingFactor", vertexScalingFactor);
 
-    double edgeThicknessScalingFactor = 6.;
+    double edgeThicknessScalingFactor = 1.;
     getParameterValue(request, "edgeThicknessScalingFactor", edgeThicknessScalingFactor);
 
-    double edgeArrowScalingFactor = 1.;
-    getParameterValue(request, "edgeArrowScalingFactor", edgeArrowScalingFactor);
-
-    string format = "svg";
-    getParameterValue(request, "format", format);
-
     double timeout = 30;
     getParameterValue(request, "timeout", timeout);
 
     string addBlastAnnotationsString;
     const bool addBlastAnnotations = getParameterValue(request, "addBlastAnnotations", addBlastAnnotationsString);
 
-    string saveDotFileString;
-    const bool saveDotFile = getParameterValue(request, "saveDotFile", saveDotFileString);
-
 
     // Write the form.
     string readGraphHeading;
@@ -125,7 +145,7 @@ void Assembler::exploreUndirectedReadGraph(
         readGraphHeading = "<h3>Display a local subgraph of the global alignment graph</h3>";
     } else {
         readGraphHeading =
-            "<h3>Display a local subgraph of the <a href='docs/ComputationalMethods.html#ReadGraph'>global alignment graph</a></h3>";
+            "<h3>Display a local subgraph of the <a href='docs/ComputationalMethods.html#ReadGraph'>read graph</a></h3>";
     }
     html << readGraphHeading << 
          "<form>"
@@ -151,12 +171,6 @@ void Assembler::exploreUndirectedReadGraph(
          " value='" << maxDistance <<
          "'>"
 
-         "<tr title='Maximum trim (markers) used to define containment'>"
-         "<td>Maximum trim"
-         "<td><input type=text required name=maxTrim size=8 style='text-align:center'"
-         " value='" << maxTrim <<
-         "'>"
-
          "<tr title='Allow reads marked as chimeric to be included in the local read graph.'>"
          "<td>Allow chimeric reads"
          "<td class=centered><input type=checkbox name=allowChimericReads" <<
@@ -202,22 +216,6 @@ void Assembler::exploreUndirectedReadGraph(
          " value='" << edgeThicknessScalingFactor <<
          "'>"
 
-         "<tr>"
-         "<td>Edge arrow scaling factor"
-         "<td><input type=text required name=edgeArrowScalingFactor size=8 style='text-align:center'" <<
-         " value='" << edgeArrowScalingFactor <<
-         "'>"
-
-         "<tr>"
-         "<td>Graphics format"
-         "<td class=centered>"
-         "<input type=radio required name=format value='svg'" <<
-         (format == "svg" ? " checked=on" : "") <<
-         ">svg"
-         "<br><input type=radio required name=format value='png'" <<
-         (format == "png" ? " checked=on" : "") <<
-         ">png"
-
          "<tr title='Maximum time (in seconds) allowed for graph creation and layout'>"
          "<td>Timeout (seconds) for graph layout"
          "<td><input type=text required name=timeout size=8 style='text-align:center'" <<
@@ -230,11 +228,6 @@ void Assembler::exploreUndirectedReadGraph(
          (addBlastAnnotations ? " checked" : "") <<
          ">"
 
-         "<tr title='Save the Graphviz dot file representing this local read graph'>"
-         "<td>Save the Graphviz dot file"
-         "<td class=centered><input type=checkbox name=saveDotFile" <<
-         (saveDotFile ? " checked" : "") <<
-         ">"
          "</table>"
          "</div>"
          "</div>"
@@ -265,15 +258,13 @@ void Assembler::exploreUndirectedReadGraph(
 
     // Create the local read graph.
     LocalReadGraph graph;
-    const auto createStartTime = steady_clock::now();
     if(!createLocalReadGraph(readIds,
-        maxDistance, allowChimericReads, allowCrossStrandEdges, maxTrim, timeout, graph)) {
+        maxDistance, allowChimericReads, allowCrossStrandEdges, timeout, graph)) {
         html << "<p>Timeout for graph creation exceeded. Increase the timeout or reduce the maximum distance from the start vertex.";
         return;
     }
     html << "<p>The local read graph has " << num_vertices(graph);
     html << " vertices and " << num_edges(graph) << " edges.";
-    const auto createFinishTime = steady_clock::now();
 
 
 
@@ -368,181 +359,40 @@ void Assembler::exploreUndirectedReadGraph(
     }
 
 
-    // Write it out in graphviz format.
-    const string uuid = to_string(boost::uuids::random_generator()());
-    const string dotFileName = tmpDirectory() + uuid + ".dot";
 
-    graph.write(dotFileName,
-                layoutMethod,
-                maxDistance,
-                vertexScalingFactor,
-                edgeThicknessScalingFactor,
-                edgeArrowScalingFactor,
-                httpServerData.assemblerOptions->alignOptions.maxTrim);
+    // Write a title.
+    html <<
+         "<h1 style='line-height:10px'>Read graph near oriented read(s) " << readIdsString << "</h1>";
 
 
-
-    // Display the graph in svg format.
-    if(format == "svg") {
-        const string command =
-            timeoutCommand() + " " + to_string(timeout - seconds(createFinishTime - createStartTime)) +
-            " dot -O -T svg " + dotFileName +
-            " -Gsize=" + to_string(sizePixels/72.);
-        const int commandStatus = ::system(command.c_str());
-        if(WIFEXITED(commandStatus)) {
-            const int exitStatus = WEXITSTATUS(commandStatus);
-            if(exitStatus == 124) {
-                html << "<p>Timeout for graph layout exceeded. Increase the timeout or reduce the maximum distance from the start vertex.";
-                filesystem::remove(dotFileName);
-                return;
-            }
-            else if(exitStatus!=0 && exitStatus!=1) {    // sfdp returns 1 all the time just because of the message about missing triangulation.
-                filesystem::remove(dotFileName);
-                throw runtime_error("Error " + to_string(exitStatus) + " running graph layout command: " + command);
-            }
-        } else if(WIFSIGNALED(commandStatus)) {
-            const int signalNumber = WTERMSIG(commandStatus);
-            throw runtime_error("Signal " + to_string(signalNumber) + " while running graph layout command: " + command);
-        } else {
-            throw runtime_error("Abnormal status " + to_string(commandStatus) + " while running graph layout command: " + command);
-
-        }
-        // Remove the .dot file.
-        if(!saveDotFile) {
-            filesystem::remove(dotFileName);
+    // Allow manually highlighting selected vertices.
+    html << R"stringDelimiter(
+        <script>
+        function highlight_vertex()
+        {
+            vertex = document.getElementById("highlight").value;
+            document.getElementById("highlight").value = "";
+            element = document.getElementById("Vertex-" + vertex);
+            element.setAttribute("fill", "#ff00ff");
         }
+        </script>
+        <p>
+        <input id=highlight type=text onchange="highlight_vertex()" size=10>
+        Enter an oriented read to highlight, then press Enter. The oriented read should be
+        in the form <code>readId-strand</code> where strand is 0 or 1 (for example, <code>"1345871-1</code>").
+        To highlight multiple oriented reads, enter them one at a time in the same way.
+        <p>
+        )stringDelimiter";
+
+    // Buttons to resize the svg locally.
+    addScaleSvgButtons(html);
+
+    // Write the graph to svg directly, without using Graphviz rendering.
+    graph.computeLayout(layoutMethod, timeout);
+    graph.writeSvg("svg", sizePixels, sizePixels,
+        vertexScalingFactor, edgeThicknessScalingFactor, maxDistance, html);
 
 
-
-        // Write a title.
-        html <<
-             "<h1 style='line-height:10px'>Read graph near oriented read(s) " << readIdsString << "</h1>"
-             "Color legend: "
-             "<span style='background-color:green'>start vertex</span> "
-             "<span style='background-color:cyan'>vertices at maximum distance (" << maxDistance <<
-             ") from the start vertex</span> "
-             ".<br>";
-
-        if(saveDotFile) {
-            html << "<p>Graphviz dot file saved as " << dotFileName << "<br>";
-        }
-
-        // Allow manually highlighting selected vertices.
-        html << R"stringDelimiter(
-            <script>
-            function highlight_vertex()
-            {
-                vertex = document.getElementById("highlight").value;
-                document.getElementById("highlight").value = "";
-                element = document.getElementById("Vertex-" + vertex);
-                ellipse = element.children[1].children[0].children[0];
-                ellipse.setAttribute("fill", "#ff00ff");
-                ellipse.setAttribute("stroke", "#ff00ff");
-            }
-            </script>
-            <p>
-            <input id=highlight type=text onchange="highlight_vertex()" size=10>
-            Enter an oriented read to highlight, then press Enter. The oriented read should be
-            in the form <code>readId-strand</code> where strand is 0 or 1 (for example, <code>"1345871-1</code>").
-            To highlight multiple oriented reads, enter them one at a time in the same way.
-            <p>
-            )stringDelimiter";
-
-        // Display the graph.
-        const string svgFileName = dotFileName + ".svg";
-        ifstream svgFile(svgFileName);
-        html << svgFile.rdbuf();
-        svgFile.close();
-
-
-
-        // Add to each vertex a cursor that shows you can click on it.
-        html <<
-            "<script>"
-            "var vertices = document.getElementsByClassName('node');"
-            "for (var i=0;i<vertices.length; i++) {"
-            "    vertices[i].style.cursor = 'pointer';"
-            "}"
-            "</script>";
-
-
-
-        // Remove the .svg file.
-        filesystem::remove(svgFileName);
-    }
-
-
-
-    // Display the graph in png format.
-    else if(format == "png") {
-
-        // Run graphviz to create the png file and the cmapx file.
-        // The cmapx file is used to create links on the image.
-        // See here for more information:
-        // https://www.graphviz.org/doc/info/output.html#d:imap
-        const string command =
-            timeoutCommand() + " " + to_string(timeout - seconds(createFinishTime - createStartTime)) +
-            " dot -O -T png -T cmapx " + dotFileName +
-            " -Gsize=" + to_string(sizePixels/72.);
-        const int commandStatus = ::system(command.c_str());
-        if(WIFEXITED(commandStatus)) {
-            const int exitStatus = WEXITSTATUS(commandStatus);
-            if(exitStatus == 124) {
-                html << "<p>Timeout for graph layout exceeded. Increase the timeout or reduce the maximum distance from the start vertex.";
-                filesystem::remove(dotFileName);
-                return;
-            }
-            else if(exitStatus!=0 && exitStatus!=1) {    // sfdp returns 1 all the time just because of the message about missing triangulation.
-                filesystem::remove(dotFileName);
-                throw runtime_error("Error " + to_string(exitStatus) + " running graph layout command: " + command);
-            }
-        } else if(WIFSIGNALED(commandStatus)) {
-            const int signalNumber = WTERMSIG(commandStatus);
-            filesystem::remove(dotFileName);
-            throw runtime_error("Signal " + to_string(signalNumber) + " while running graph layout command: " + command);
-        } else {
-            filesystem::remove(dotFileName);
-            throw runtime_error("Abnormal status " + to_string(commandStatus) + " while running graph layout command: " + command);
-
-        }
-        // Remove the .dot file.
-        if(!saveDotFile) {
-            filesystem::remove(dotFileName);
-        } else {
-            html << "<p>Graphviz dot file saved as " << dotFileName;
-        }
-
-        // Get the names of the files we created.
-        const string pngFileName = dotFileName + ".png";
-        const string cmapxFileName = dotFileName + ".cmapx";
-
-        // Create a base64 version of the png file.
-        const string base64FileName = pngFileName + ".base64";
-        const string base64Command = "base64 " + pngFileName + " > " +
-            base64FileName;
-        ::system(base64Command.c_str());
-
-
-        // Write out the png image.
-        html << "<p><img usemap='#G' src=\"data:image/png;base64,";
-        ifstream png(base64FileName);
-        html << png.rdbuf();
-        html << "\"/>";
-        ifstream cmapx(cmapxFileName);
-        html << cmapx.rdbuf();
-
-        // Remove the files we created.
-        filesystem::remove(pngFileName);
-        filesystem::remove(cmapxFileName);
-        filesystem::remove(base64FileName);
-    }
-
-
-
-    // If got here, the format string is not one of the ones
-    // we support.
-    else {
-        html << "Invalid format " << format << " specified";
-        filesystem::remove(dotFileName);
-    }
 }
+
+#endif
