@@ -10,6 +10,8 @@ using namespace shasta;
 #include <boost/uuid/uuid_generators.hpp>
 #include <boost/uuid/uuid_io.hpp>
 
+
+
 #ifdef SHASTA_HTTP_SERVER
 
 #define SHASTA_ADD_TO_FUNCTION_TABLE(name) httpServerData.functionTable[string("/") + #name ] = &Assembler::name
@@ -30,7 +32,8 @@ void Assembler::fillServerFunctionTable()
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreAlignment);
     SHASTA_ADD_TO_FUNCTION_TABLE(computeAllAlignments);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreAlignmentGraph);
-    SHASTA_ADD_TO_FUNCTION_TABLE(displayAlignmentMatrix);
+    SHASTA_ADD_TO_FUNCTION_TABLE(alignSequencesInBaseRepresentation);
+    SHASTA_ADD_TO_FUNCTION_TABLE(alignSequencesInMarkerRepresentation);
     SHASTA_ADD_TO_FUNCTION_TABLE(assessAlignments);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreReadGraph);
     SHASTA_ADD_TO_FUNCTION_TABLE(exploreMarkerGraph);
@@ -64,6 +67,7 @@ void Assembler::processRequest(
         // If it contains "/", reject it.
         if(name.find('/') != string::npos) {
             writeHtmlBegin(html);
+            writeNavigation(html);
             html << "Unknown documentation file " << name;
             writeHtmlEnd(html);
             return;
@@ -74,6 +78,7 @@ void Assembler::processRequest(
         ifstream file(fileName);
         if(!file) {
             writeHtmlBegin(html);
+            writeNavigation(html);
             html << "Could not open " << fileName;
             writeHtmlEnd(html);
         }
@@ -90,6 +95,7 @@ void Assembler::processRequest(
     const auto it = httpServerData.functionTable.find(keyword);
     if(it == httpServerData.functionTable.end()) {
         writeHtmlBegin(html);
+        writeNavigation(html);
         html << "Unsupported keyword " << keyword;
         writeHtmlEnd(html);
         return;
@@ -99,6 +105,7 @@ void Assembler::processRequest(
     // We found the keyword. Call the function that processes this keyword.
     // The processing function is only responsible for writing the html body.
     writeHtmlBegin(html);
+    writeNavigation(html);
     try {
         const auto function = it->second;
         (this->*function)(request, html);
@@ -111,148 +118,54 @@ void Assembler::processRequest(
 
 
 
-void Assembler::writeStyle(ostream& html)
+void Assembler::writeMakeAllTablesCopyable(ostream& html) const
 {
-    html << R"%(
-<style>
-    body {
-        font-family: Arial;
-    }
-    pre {
-        font-family: courier;
-    }
-    p, input {
-        font-size: 16px;
-    }
-    h1, h2, h3 {
-        color: DarkSlateBlue;
-    }
-    table {
-        border-collapse: collapse;
-    }
-    th, td {
-        border: 1px solid #b8b5c7d9;
-        padding: 2px;
-    }
-    th {
-        font-weight: bold;
-        text-align: center;
-    }
-    th.left {
-        text-align: left;
-    }
-    td.centered {
-        text-align: center;
-    }
-    td.right {
-        text-align: right;
-    }
-    td.smaller {
-        font-size: smaller;
-    }
-    a {
-        color: DarkSlateBlue;
-    }
-    ul.navigationMenu {
-        list-style-type: none;
-        margin: 0px 0px 12px 0px;
-        padding: 0;
-        overflow: hidden;
-        background-color: #404040;
-    }
-    
-    div.navigationButton {
-        display: inline-block;
-        color: white;
-        text-align: center;
-        padding: 14px 16px;
-        text-decoration: none;
-        // min-width: 120px;
-    }
-    
-    .navigationMenuEntry:hover .navigationButton {
-        background-color: black;
-    }
-    
-    li.navigationMenuEntry {
-        display: inline-block;
-    }
-    
-    .navigationItems {
-        display: none;
-        position: absolute;
-        background-color: DodgerBlue;
-        // min-width: 120px;
-        box-shadow: 0px 8px 16px 0px rgba(0,0,0,0.2);
-        z-index: 1;
-    }
-    
-    a.navigationItem {
-        color: black;
-        padding: 12px 16px;
-        text-decoration: none;
-        display: block;
-        text-align: left;
-    }
-    
-    .navigationItems a:hover {background-color: SteelBlue}
-    
-    .navigationMenuEntry:hover .navigationItems {
-        display: block;
+    html << R"###(
+    <script>
+
+    // Copy to the clipboard the table that generated the event.
+    function copyToClipboard(event)
+    {
+        // Turn off the standard context menu.
+        event.preventDefault();
+        
+        // Get the table element.
+        var element = event.currentTarget;
+         
+        // Remove any previous selection.
+        var selection = window.getSelection();
+        selection.removeAllRanges();
+        
+        // Select the table.
+        var range = document.createRange();
+        range.selectNodeContents(element);
+        selection.addRange(range);
+        
+        // Copy it to the clipboard.
+        document.execCommand("copy");
     }
 
-    input[type=submit] {
-        background-color: #89bef2;
-        padding: 4px;
+    // Make a table copyable by right click.
+    function makeCopyable(element)
+    {
+        element.addEventListener('contextmenu', copyToClipboard);
+        element.title = 'Right click anywhere on the table to copy the entire table to the clipboard';
     }
 
-    input[type=button] {
-        padding: 4px;
+    // Make all tables copyable by right click.
+    function makeAllTablesCopyable()
+    {
+        var tables = document.getElementsByTagName('table');
+        var i;
+        for(i=0; i<tables.length; i++) {
+            makeCopyable(tables[i]);
+        }
     }
-
-    input[type=text], input[type=radio] {
-        background-color: #ecf1f0;
-        border-width: thin;
-    }
-</style>
-    )%";
-}
+    </script>
+    )###";
 
 
-
-void Assembler::writeHtmlBegin(ostream& html, bool navigation) const
-{
-    html <<
-        "\r\n"
-        "<!DOCTYPE html>"
-        "<html>"
-        "<head>"
-        "<meta charset='UTF-8'>"
-        "<title>Shasta assembler</title>";
-    writeStyle(html);
-    // writeMakeAllTablesSelectable(html);
-    html <<
-        "</head>"
-        ;// "<body onload='makeAllTablesSelectableByDoubleClick()'>";
-
-    if(navigation) {
-        writeNavigation(html);
-    }
-}
-
-
-
-void Assembler::writeHtmlEnd(ostream& html) const
-{
-    html << "</body>";
-    html << "</html>";
-}
-
-
-
-#ifdef SHASTA_HTTP_SERVER
-void Assembler::writeMakeAllTablesSelectable(ostream& html) const
-{
+#if 0
     html << R"###(
 <script>
 
@@ -286,11 +199,14 @@ function makeAllTablesSelectableByDoubleClick()
 }
 </script>
     )###";
-}
 #endif
+}
 
 
 
+#ifdef SHASTA_HTTP_SERVER
+
+
 void Assembler::writeNavigation(ostream& html) const
 {
     html << "<ul class=navigationMenu>";
@@ -306,8 +222,9 @@ void Assembler::writeNavigation(ostream& html) const
         {"Align two reads", "exploreAlignment"},
         {"Align one read with all", "computeAllAlignments"},
         {"Alignment graph", "exploreAlignmentGraph"},
-        {"Alignment matrix", "displayAlignmentMatrix"},
         {"Assess alignments", "assessAlignments"},
+        {"Align sequences in base representation", "alignSequencesInBaseRepresentation"},
+        // {"Align sequences in marker representation", "alignSequencesInMarkerRepresentation"},
         });
     writeNavigation(html, "Read graph", {
         {"Read graph", "exploreReadGraph"},
@@ -425,7 +342,16 @@ void Assembler::writeGnuPlotPngToHtml(
 }
 
 
-#ifdef SHASTA_HTTP_SERVER
+
+void Assembler::exploreSummary(
+    const vector<string>& request,
+    ostream& html)
+{
+    writeAssemblySummaryBody(html);
+}
+
+#endif
+
 
 // Access all available assembly data, without throwing exceptions
 void Assembler::accessAllSoft()
@@ -475,19 +401,8 @@ void Assembler::accessAllSoft()
     try {
         accessReadGraph();
     } catch(const exception& e) {
-
-        // We don't have the undirected read graph. Try the directed one.
-        try {
-            accessDirectedReadGraphReadOnly();
-        } catch(const exception& e) {
-            cout << "The read graph is not accessible." << endl;
-            allDataAreAvailable = false;
-        }
-
-        try {
-            accessConflictReadGraph();
-        } catch(const exception& e) {
-        }
+        cout << "The read graph is not accessible." << endl;
+        allDataAreAvailable = false;
     }
 
 
@@ -541,14 +456,6 @@ void Assembler::accessAllSoft()
         allDataAreAvailable = false;
     }
 
-    try {
-        accessPhasingData();
-    } catch(const exception& e) {
-    	// Don't threat it as missing because this does not get created in all cases.
-        // cout << "Assembly graph sequences are not accessible." << endl;
-        // allDataAreAvailable = false;
-    }
-
     try {
         accessCompressedAlignments();
     } catch(const exception& e) {
@@ -564,21 +471,156 @@ void Assembler::accessAllSoft()
 
 
 
-void Assembler::exploreSummary(
-    const vector<string>& request,
-    ostream& html)
+
+void Assembler::writeStyle(ostream& html)
 {
-    writeAssemblySummaryBody(html);
+    html << R"%(
+<style>
+    body {
+        font-family: Arial;
+    }
+    pre {
+        font-family: courier;
+    }
+    p, input {
+        font-size: 16px;
+    }
+    h1, h2, h3 {
+        color: DarkSlateBlue;
+    }
+    table {
+        border-collapse: collapse;
+    }
+    th, td {
+        border: 1px solid #b8b5c7d9;
+        padding: 2px;
+    }
+    th {
+        font-weight: bold;
+        text-align: center;
+    }
+    th.left {
+        text-align: left;
+    }
+    td.centered {
+        text-align: center;
+    }
+    td.right {
+        text-align: right;
+    }
+    td.smaller {
+        font-size: smaller;
+    }
+    a {
+        color: DarkSlateBlue;
+    }
+    ul.navigationMenu {
+        list-style-type: none;
+        margin: 0px 0px 12px 0px;
+        padding: 0;
+        overflow: hidden;
+        background-color: #404040;
+    }
+    
+    div.navigationButton {
+        display: inline-block;
+        color: white;
+        text-align: center;
+        padding: 14px 16px;
+        text-decoration: none;
+        // min-width: 120px;
+    }
+    
+    .navigationMenuEntry:hover .navigationButton {
+        background-color: black;
+    }
+    
+    li.navigationMenuEntry {
+        display: inline-block;
+    }
+    
+    .navigationItems {
+        display: none;
+        position: absolute;
+        background-color: DodgerBlue;
+        // min-width: 120px;
+        box-shadow: 0px 8px 16px 0px rgba(0,0,0,0.2);
+        z-index: 1;
+    }
+    
+    a.navigationItem {
+        color: black;
+        padding: 12px 16px;
+        text-decoration: none;
+        display: block;
+        text-align: left;
+    }
+    
+    .navigationItems a:hover {background-color: SteelBlue}
+    
+    .navigationMenuEntry:hover .navigationItems {
+        display: block;
+    }
+
+    input[type=submit] {
+        background-color: #89bef2;
+        padding: 4px;
+        margin: 2px;
+        border-radius: 8px;
+    }
+
+    input[type=button] {
+        padding: 4px;
+    }
+
+    input[type=text], input[type=radio] {
+        background-color: #ecf1f0;
+        border-width: thin;
+    }
+
+    button {
+        background-color: #89bef2;
+        padding: 4px;
+        margin: 2px;
+        border-radius: 8px;
+    }
+
+</style>
+    )%";
+}
+
+
+void Assembler::writeHtmlBegin(ostream& html) const
+{
+    html <<
+        "\r\n"
+        "<!DOCTYPE html>"
+        "<html>"
+        "<head>"
+        "<meta charset='UTF-8'>"
+        "<title>Shasta assembler</title>";
+    writeStyle(html);
+    writeMakeAllTablesCopyable(html);
+    html <<
+        "</head>"
+        "<body onload='makeAllTablesCopyable()'>";
+}
+
+
+
+void Assembler::writeHtmlEnd(ostream& html) const
+{
+    html << "</body>";
+    html << "</html>";
 }
 
-#endif
 
 
 
 
 void Assembler::writeAssemblySummary(ostream& html)
 {
-    writeHtmlBegin(html, false);
+    writeHtmlBegin(html);
     writeAssemblySummaryBody(html);
     writeHtmlEnd(html);
 }
@@ -724,9 +766,7 @@ void Assembler::writeAssemblySummaryBody(ostream& html)
         "<tr><td>Number of good alignments"
         "<td class=right>" << alignmentData.size() <<
         "<tr><td>Number of good alignments kept in the read graph"
-        "<td class=right>" << (directedReadGraph.isOpen() ?
-            directedReadGraph.edges.size()/2 :
-            readGraph.edges.size()/2) <<
+        "<td class=right>" << readGraph.edges.size()/2 <<
         "</table>"
 
 
@@ -734,13 +774,9 @@ void Assembler::writeAssemblySummaryBody(ostream& html)
         "<h3>Read graph</h3>"
         "<table>"
         "<tr><td>Number of vertices"
-        "<td class=right>" << (directedReadGraph.isOpen() ?
-            directedReadGraph.vertices.size() :
-            readGraph.connectivity.size()) <<
+        "<td class=right>" << readGraph.connectivity.size() <<
         "<tr><td>Number of edges"
-        "<td class=right>" << (directedReadGraph.isOpen() ?
-            directedReadGraph.edges.size() :
-            readGraph.edges.size()) <<
+        "<td class=right>" << readGraph.edges.size() <<
         "</table>"
         "<ul>"
         "<li>The read graph contains both strands. Each read generates two vertices."
@@ -962,21 +998,15 @@ void Assembler::writeAssemblySummaryJson(ostream& json)
         "    \"Number of alignment candidates found by the LowHash algorithm\": " <<
         alignmentCandidates.candidates.size() << ",\n"
         "    \"Number of good alignments\": " << alignmentData.size() << ",\n"
-        "    \"Number of good alignments kept in the read graph\": " << (directedReadGraph.isOpen() ?
-            directedReadGraph.edges.size()/2 :
-            readGraph.edges.size()/2) << "\n"
+        "    \"Number of good alignments kept in the read graph\": " << readGraph.edges.size()/2 << "\n"
         "  },\n"
 
 
 
         "  \"Read graph\":\n"
         "  {\n"
-        "    \"Number of vertices\": " << (directedReadGraph.isOpen() ?
-            directedReadGraph.vertices.size() :
-            readGraph.connectivity.size()) << ",\n"
-        "    \"Number of edges\": " << (directedReadGraph.isOpen() ?
-            directedReadGraph.edges.size() :
-            readGraph.edges.size()) << ",\n"
+        "    \"Number of vertices\": " << readGraph.connectivity.size() << ",\n"
+        "    \"Number of edges\": " << readGraph.edges.size() << ",\n"
         "    \"Isolated reads\":\n"
         "    {\n"
         "      \"Reads\": " << assemblerInfo->isolatedReadCount << ",\n"
@@ -1051,7 +1081,7 @@ void Assembler::writeAssemblySummaryJson(ostream& json)
 
 void Assembler::writeAssemblyIndex(ostream& html) const
 {
-    writeHtmlBegin(html, false);
+    writeHtmlBegin(html);
 
     const string s = R"ABCDE(
     <body>
