@@ -45,10 +45,17 @@ void Assembler::createReadGraph(
     // Contains pairs(marker count, alignment id).
     vector< pair<uint32_t, uint32_t> > readAlignments;
 
+    const bool debug = false;
+    if(debug) {
+        cout << "createReadGraph begins, maxAlignmentCount " << maxAlignmentCount << endl;
+    }
 
 
     // Loop over reads.
     for(ReadId readId=0; readId<readCount; readId++) {
+        if(debug) {
+            cout << "Working on read " << readId << endl;
+        }
 
         // Gather the alignments for this read, each with its number of markers.
         readAlignments.clear();
@@ -56,6 +63,9 @@ void Assembler::createReadGraph(
             const AlignmentData& alignment = alignmentData[alignmentId];
             readAlignments.push_back(make_pair(alignment.info.markerCount, alignmentId));
         }
+        if(debug) {
+            cout << "Found " << readAlignments.size() << " alignments." << endl;
+        }
 
         // Keep the best maxAlignmentCount.
         if(readAlignments.size() > maxAlignmentCount) {
@@ -66,11 +76,19 @@ void Assembler::createReadGraph(
                 std::greater< pair<uint32_t, uint32_t> >());
             readAlignments.resize(maxAlignmentCount);
         }
+        if(debug) {
+            cout << "Kept " << readAlignments.size() << " alignments." << endl;
+        }
 
         // Mark the surviving alignments as to be kept.
         for(const auto& p: readAlignments) {
             const uint32_t alignmentId = p.second;
             keepAlignment[alignmentId] = true;
+            if(debug) {
+                const AlignmentData& alignment = alignmentData[alignmentId];
+                cout << "Marked alignment " << alignment.readIds[0] << " " <<
+                    alignment.readIds[1] << (alignment.isSameStrand ? " same strand" : " opposite strand") << endl;
+            }
         }
     }
     const size_t keepCount = count(keepAlignment.begin(), keepAlignment.end(), true);
@@ -177,7 +195,6 @@ bool Assembler::createLocalReadGraph(
         uint32_t maxDistance,           // How far to go from starting oriented read.
         bool allowChimericReads,
         bool allowCrossStrandEdges,
-        size_t maxTrim,                 // Used to define containment.
         double timeout,                 // Or 0 for no timeout.
         LocalReadGraph& graph)
 {
@@ -187,7 +204,6 @@ bool Assembler::createLocalReadGraph(
             maxDistance,           // How far to go from starting oriented read.
             allowChimericReads,
             allowCrossStrandEdges,
-            maxTrim,                 // Used to define containment.
             timeout,                 // Or 0 for no timeout.
             graph
     );
@@ -204,7 +220,6 @@ bool Assembler::createLocalReadGraph(
     uint32_t maxDistance,           // How far to go from starting oriented read.
     bool allowChimericReads,
     bool allowCrossStrandEdges,
-    size_t maxTrim,                 // Used to define containment.
     double timeout,                 // Or 0 for no timeout.
     LocalReadGraph& graph)
 {
@@ -274,7 +289,6 @@ bool Assembler::createLocalReadGraph(
                 alignmentInfo.reverseComplement();
             }
             SHASTA_ASSERT(alignmentOrientedReadId0 == orientedReadId0);
-            const AlignmentType alignmentType = alignmentInfo.classify(uint32_t(maxTrim));
             const uint32_t markerCount = alignmentInfo.markerCount;
 
             // Update our BFS.
@@ -291,7 +305,6 @@ bool Assembler::createLocalReadGraph(
                         orientedReadId0,
                         orientedReadId1,
                         markerCount,
-                        alignmentType,
                         globalEdge.crossesStrands == 1);
             } else {
                 SHASTA_ASSERT(distance0 == maxDistance);
@@ -300,7 +313,6 @@ bool Assembler::createLocalReadGraph(
                             orientedReadId0,
                             orientedReadId1,
                             markerCount,
-                            alignmentType,
                             globalEdge.crossesStrands == 1);
                 }
             }
@@ -738,7 +750,6 @@ void Assembler::writeLocalReadGraphReads(
         maxDistance,
         allowChimericReads,
         allowCrossStrandEdges,
-        std::numeric_limits<size_t>::max(),
         0.,
         localReadGraph));
 
