@@ -221,6 +221,22 @@ PYBIND11_MODULE(shasta, module)
             arg("matchCount"),
             arg("mismatchCount"),
             arg("gapCount"))
+        .def("alignOrientedReads4",
+            (
+                void (Assembler::*)
+                (ReadId, Strand, ReadId, Strand, uint64_t, uint64_t, uint64_t, int64_t, int64_t, int64_t) const
+            )
+            &Assembler::alignOrientedReads4,
+            arg("readId0"),
+            arg("strand0"),
+            arg("readId1"),
+            arg("strand1"),
+            arg("m"),
+            arg("deltaX"),
+            arg("deltaY"),
+            arg("matchScore"),
+            arg("mismatchScore"),
+            arg("gapScore"))
         .def("alignOverlappingOrientedReads",
             (
                 void (Assembler::*)
@@ -265,10 +281,6 @@ PYBIND11_MODULE(shasta, module)
             arg("strand0"),
             arg("readId1"),
             arg("strand1"))
-        .def("analyzeAlignments",
-            &Assembler::analyzeAlignments,
-            arg("readId"),
-            arg("strand"))
 
 
 
@@ -327,26 +339,6 @@ PYBIND11_MODULE(shasta, module)
 
 
 
-        // Directed read graph.
-        .def("createDirectedReadGraph",
-            &Assembler::createDirectedReadGraph,
-            arg("maxTrim"),
-            arg("containedNeighborCount"),
-            arg("uncontainedNeighborCountPerDirection"))
-        .def("accessDirectedReadGraphReadOnly",
-            &Assembler::accessDirectedReadGraphReadOnly)
-        .def("accessDirectedReadGraphReadWrite",
-            &Assembler::accessDirectedReadGraphReadWrite)
-        .def("markDirectedReadGraphConflictEdges1",
-            &Assembler::markDirectedReadGraphConflictEdges1)
-        .def("markDirectedReadGraphConflictEdges2",
-            &Assembler::markDirectedReadGraphConflictEdges2,
-            arg("radius"))
-        .def("markDirectedReadGraphConflictEdges3",
-            &Assembler::markDirectedReadGraphConflictEdges3,
-            arg("radius"))
-
-
         // Global marker graph.
         .def("createMarkerGraphVertices",
             &Assembler::createMarkerGraphVertices,
@@ -360,7 +352,6 @@ PYBIND11_MODULE(shasta, module)
             arg("downsamplingFactor"),
             arg("bandExtend"),
             arg("maxBand"),
-            arg("readGraphCreationMethod"),
             arg("minCoverage"),
             arg("maxCoverage"),
             arg("minCoveragePerStrand"),
@@ -419,8 +410,6 @@ PYBIND11_MODULE(shasta, module)
             arg("threadCount") = 0)
         .def("computeMarkerGraphCoverageHistogram",
             &Assembler::computeMarkerGraphCoverageHistogram)
-        .def("analyzeMarkerGraphVertex",
-            &Assembler::analyzeMarkerGraphVertex)
         .def("refineMarkerGraph",
             &Assembler::refineMarkerGraph,
             arg("refineThreshold"),
@@ -470,17 +459,7 @@ PYBIND11_MODULE(shasta, module)
             &Assembler::accessMarkerGraphConsensus)
         .def("accessMarkerGraphCoverageData",
             &Assembler::accessMarkerGraphCoverageData)
-        .def("createConflictReadGraph",
-            &Assembler::createConflictReadGraph,
-            arg("threadCount") = 0,
-            arg("maxOffsetSigma"),
-            arg("maxTrim"),
-            arg("maxSkip"),
-            arg("minAlignedMarkerCount"))
-        .def("accessConflictReadGraph",
-            &Assembler::accessConflictReadGraph)
-        .def("cleanupConflictReadGraph",
-            &Assembler::cleanupConflictReadGraph)
+
 
 
         // Assembly graph.
@@ -561,29 +540,11 @@ PYBIND11_MODULE(shasta, module)
             arg("diagonalReadCountMin"),
             arg("offDiagonalReadCountMax"),
             arg("offDiagonalRatio"))
-        .def("createSegmentGraph",
-            &Assembler::createSegmentGraph)
-        .def("colorGfaBySegmentGraphChain",
-            &Assembler::colorGfaBySegmentGraphChain)
-        .def("analyzeOrientedReadPaths",
-            &Assembler::analyzeOrientedReadPaths)
-        .def("analyzeOrientedReadPathsThroughSegment",
-            (
-                void (Assembler::*)
-                (AssemblyGraph::EdgeId)
-            )
-            &Assembler::analyzeOrientedReadPathsThroughSegment,
-            arg("segmentId"))
-        .def("followOrientedReadPaths",
-            &Assembler::followOrientedReadPaths,
-            arg("segmentId"),
-            arg("forward"))
         .def("alignPseudoPaths",
             &Assembler::alignPseudoPaths)
 
 
 
-
             // Consensus caller.
         .def("setupConsensusCaller",
             &Assembler::setupConsensusCaller)
@@ -595,45 +556,9 @@ PYBIND11_MODULE(shasta, module)
             &Assembler::colorCompressedAssemblyGraph,
             arg("gfaId"))
 
-        // Phasing.
-        .def("createPhasingData",
-            &Assembler::createPhasingData,
-            arg("threadCount") = 0,
-            arg("phasingSimilarityThreshold"),
-            arg("maxNeighborCount"))
-        .def("accessPhasingData",
-            &Assembler::accessPhasingData)
-#if 0
-        .def("computePhasingSimilarity",
-            (
-                double (Assembler::*)
-                (ReadId, Strand, ReadId, Strand)
-            )
-            &Assembler::computePhasingSimilarity,
-            arg("readId0"),
-            arg("strand0"),
-            arg("readId1"),
-            arg("strand1")
-            )
-#endif
-        .def("computePhasingSimilarity",
-            (
-                double (Assembler::*)
-                (AssemblyGraph::EdgeId, AssemblyGraph::EdgeId)
-            )
-            &Assembler::computePhasingSimilarity,
-            arg("edgeId0"),
-            arg("edgeId1")
-            )
-        .def("countCommonInternalOrientedReads",
-            &Assembler::countCommonInternalOrientedReads,
-            arg("edgeId0"),
-            arg("edgeId1")
-            )
 
         .def("test", &Assembler::test)
 
-
         // Definition of class_Assembler ends here.
     ;
 
