@@ -41,9 +41,11 @@ namespace shasta {
         void assemble(const AssemblerOptions&);
         void saveBinaryData(const AssemblerOptions&);
         void cleanupBinaryData(const AssemblerOptions&);
-        void explore(const AssemblerOptions&);
         void createBashCompletionScript(const AssemblerOptions&);
 
+#ifdef SHASTA_HTTP_SERVER
+        void explore(const AssemblerOptions&);
+#endif
     }
 }
 using namespace shasta;
@@ -132,7 +134,11 @@ void shasta::main::main(int argumentCount, const char** arguments)
         saveBinaryData(assemblerOptions);
         return;
     } else if(assemblerOptions.commandLineOnlyOptions.command == "explore") {
+#ifdef SHASTA_HTTP_SERVER
         explore(assemblerOptions);
+#else
+        throw runtime_error("Shasta was not built with HTTP server support.");
+#endif
         return;
     } else if(assemblerOptions.commandLineOnlyOptions.command == "createBashCompletionScript") {
         createBashCompletionScript(assemblerOptions);
@@ -694,11 +700,6 @@ void shasta::main::assemble(
         // Flag chimeric reads.
         assembler.flagChimericReads(assemblerOptions.readGraphOptions.maxChimericReadDistance, threadCount);
         assembler.computeReadGraphConnectedComponents(assemblerOptions.readGraphOptions.minComponentSize);
-    } else if(assemblerOptions.readGraphOptions.creationMethod == 1) {
-        assembler.createDirectedReadGraph(
-            assemblerOptions.alignOptions.maxTrim,
-            assemblerOptions.readGraphOptions.containedNeighborCount,
-            assemblerOptions.readGraphOptions.uncontainedNeighborCountPerDirection);
     } else if(assemblerOptions.readGraphOptions.creationMethod == 2) {
         assembler.createReadGraph2(
             assemblerOptions.readGraphOptions.maxAlignmentCount,
@@ -722,40 +723,6 @@ void shasta::main::assemble(
 
 
 
-    // Optional removal of conflicts from the read graph (experimental).
-    if(assemblerOptions.readGraphOptions.removeConflicts) {
-
-        // This only works when using the directed read graph.
-        SHASTA_ASSERT(assembler.directedReadGraph.isOpen());
-
-        // Preliminary creation of marker graph vertices,
-        // necessary to be able to create the conflict read graph.
-        AssemblerOptions tmpOptions = assemblerOptions;
-        tmpOptions.markerGraphOptions.minCoverage = 3;  //  ************ EXPOSE WHEN CODE STABILIZES
-        createMarkerGraphVertices(assembler, tmpOptions, threadCount);
-
-        // Create the conflict read graph.
-        // TURN THESE PARAMETERS INTO COMMAND LINE OPTIONS WHEN CODE STABILIZES. ****************
-        const uint32_t maxOffsetSigma = 100;
-        const uint32_t maxTrim = 100;
-        const uint32_t maxSkip = 100;
-        const uint32_t minAlignedMarkerCount = 100;
-        assembler.createConflictReadGraph(
-            threadCount, maxOffsetSigma, maxTrim, maxSkip,minAlignedMarkerCount);
-        assembler.cleanupConflictReadGraph();
-
-        // Mark conflict edges in the read graph.
-        // TURN THIS PARAMETER INTO A COMMAND LINE OPTION WHEN CODE STABILIZES. ****************
-        const uint32_t radius = 12;
-        assembler.markDirectedReadGraphConflictEdges3(radius);
-
-        // Remove the preliminary marker graph vertices we created.
-        assembler.removeMarkerGraphVertices();
-
-    }
-
-
-
     // Iterative assembly, if requested (experimental).
     if(assemblerOptions.assemblyOptions.iterative) {
         for(uint64_t iteration=0;
@@ -894,6 +861,11 @@ void shasta::main::assemble(
         assembler.createAssemblyGraphVertices();
     }
 
+    // Prune the assembly graph, if requested.
+    if(assemblerOptions.assemblyOptions.pruneLength > 0) {
+        assembler.pruneAssemblyGraph(assemblerOptions.assemblyOptions.pruneLength);
+    }
+
     // Detangle, if requested.
     if(assemblerOptions.assemblyOptions.detangleMethod == 1) {
         assembler.detangle();
@@ -1001,7 +973,6 @@ void shasta::main::createMarkerGraphVertices(
         assemblerOptions.alignOptions.downsamplingFactor,
         assemblerOptions.alignOptions.bandExtend,
         assemblerOptions.alignOptions.maxBand,
-        assemblerOptions.readGraphOptions.creationMethod,
         assemblerOptions.markerGraphOptions.minCoverage,
         assemblerOptions.markerGraphOptions.maxCoverage,
         assemblerOptions.markerGraphOptions.minCoveragePerStrand,
@@ -1091,12 +1062,6 @@ void shasta::main::saveBinaryData(
             " running command:\n" + command);
     }
     cout << "Binary data successfully saved." << endl;
-
-    cout << "Data are not guaranteed to be physically on disk until a sync command completes. "
-        "To guarantee that all data are physically on disk, use a sync command "
-        "and wait for it to complete, "
-        "or stop the system using a clean shutdown procedure (shutdown command), "
-        "which includes a sync." << endl;
 }
 
 
@@ -1136,7 +1101,7 @@ void shasta::main::cleanupBinaryData(
 
 }
 
-
+#ifdef SHASTA_HTTP_SERVER
 // Implementation of --command explore.
 void shasta::main::explore(
     const AssemblerOptions& assemblerOptions)
@@ -1206,6 +1171,7 @@ void shasta::main::explore(
         sameUserOnly);
 }
 
+#endif
 
 
 // This creates a bash completion script for the Shasta executable,
